@@ -2,8 +2,8 @@ post <-
 function(object, level="upper"){
  if (level=="upper" && !is.null(object$Misc$mform2)){
       data <- object$data
-      group <- object$Misc$group
-      mform2 <-object$Misc$mform2
+      mform2 <- object$Misc$mform2
+      group <- object$data[mform2][row.names(unique(data[mform2])),]
       post.prob <- object$post.prob[row.names(unique(data[mform2])),] ; dimnames(post.prob)[[1]]<- group
       post.int  <- object$post.int[row.names(unique(data[mform2]))]; names(post.int)<- group
   } else {
