@@ -1,6 +1,7 @@
 "print.glmmNPML" <-
 function(x,digits=max(3,getOption('digits')-3), ...){
   np <- length(x$coefficients)
+  # print(names(x))
   if (np > 0){   
     m <- seq(1,np)[substr(attr(x$coefficients,'names'),1,4)=='MASS']
     mass.points <- x$coefficients[m]
@@ -13,16 +14,19 @@ function(x,digits=max(3,getOption('digits')-3), ...){
     cat("No coefficients. \n")
   }
   
-  if (x$family$family=='gaussian' && x$Misc$lambda<=1/length(x$masses)){ #print sigma only if it is constant over components
-        cat('MLE of sigma:\t  ',
+  if (x$family$family=='gaussian' && x$Misc$lambda<=1/length(x$masses)){ # print sigma only if it is constant over components
+        cat('Component distribution - MLE of sigma:\t  ',                # 03/09/07
         format(signif(x$sdev$sdev,digits)),'\n')
     }
-  if (x$family$family=='Gamma'&& x$Misc$lambda<=1/length(x$masses)){ #print shape only if it is constant over components
-       cat('MLE of shape parameter:\t  ',format(signif(x$shape$shape,digits)),'\n')
+  if (x$family$family=='Gamma'&& x$Misc$lambda<=1/length(x$masses)){ # print shape only if it is constant over components
+       cat('Component distribution - MLE of shape parameter:\t  ',format(signif(x$shape$shape,digits)),'\n')# 03/09/07
     }
+
+  cat('Random effect distribution - standard deviation:\t  ', format(x$rsdev),'\n\n') # 03/09/07
+  
   if (!is.null(x$post.prob)){
       p <- x$masses
-      names(p) <- paste('MASS',seq(1,ncol(x$post.prob)),sep='')
+      # names(p) <- paste('MASS',seq(1,ncol(x$post.prob)),sep='') # omitted from 0.42 
       cat('Mixture proportions')
       cat(":\n")
       print.default(format(p,digits),print.gap=2,quote=FALSE)
